#pragma once
#include "ListItem.h"

//[U[`bZ[W
#define	WM_USER_ENDITEMEDIT		(WM_USER+530)

//---------------------------------------------------------------------------
//
//	ҏWpRg[C^[tFCX
//
//---------------------------------------------------------------------------
class CItemEditor
{
public:
	virtual ~CItemEditor(){}

	virtual int GetItemImage() const { return -1; }
	virtual int GetItemText(WTL::CString& strText) const { return 0; }

	virtual void BeginItemEdit(const CRect& rect, LPCTSTR strText, int image=-1)=0;
	virtual void EndItemEdit(){}

	virtual HWND InitItemEdit(HWND hWndParent)=0;
	virtual operator HWND() const=0;
};

//---------------------------------------------------------------------------
//
//	ҏWpGfBbg
//
//---------------------------------------------------------------------------
class CLabelEdit : public CWindowImpl<CLabelEdit, CEdit>, public CItemEditor
{
public:
	DECLARE_WND_SUPERCLASS(NULL, CEdit::GetWndClassName())

	//bZ[W}bv
	BEGIN_MSG_MAP(CLabelEdit)
		MSG_WM_KEYDOWN(OnKeyDown)
		MSG_WM_KILLFOCUS(OnKillFocus)
	END_MSG_MAP()

	// EChEnhԂ܂
	operator HWND() const
	{
		return m_hWnd;
	}
	// L[_ECxg
	void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
	{
		if ( nChar==VK_ESCAPE || nChar==VK_RETURN )
			GetParent().SetFocus();
		else
			SetMsgHandled(FALSE);
	}

	// tH[JXCxg
	void OnKillFocus(CWindow wndFocus)
	{
		::SendMessage(GetParent(), WM_USER_ENDITEMEDIT, NULL, NULL);
		SetMsgHandled(FALSE);
	}
	// ҏW̎擾
	int GetItemText(WTL::CString& strText) const
	{
		ATL::CString temp;

		GetWindowText(temp);
		strText = temp;

		return strText.GetLength();
	}
	// ҏWJn܂
	void BeginItemEdit(const CRect& rect, LPCTSTR strText, int image=-1)
	{
		//EChEʒuƃTCYݒ
		MoveWindow(&rect);
		ShowWindow(SW_SHOW);
		SetFocus();

		SetWindowText(strText);
		SetSelAll();
	}
	// ҏWI܂
	void EndItemEdit()
	{
		ShowWindow(SW_HIDE);
	}
	// EChE쐬܂
	HWND InitItemEdit(HWND hWndParent)
	{
		Create(hWndParent, NULL, _T("LabelEdit"),
			ES_MULTILINE|ES_WANTRETURN|ES_AUTOHSCROLL|ES_AUTOVSCROLL|WS_CHILD|WS_BORDER);
		SetFont( GetParent().GetFont() );

		return GetParent();
	}
};

//---------------------------------------------------------------------------
//
//	ҏWpR{{bNX
//
//---------------------------------------------------------------------------
class CComboEdit : public CWindowImpl<CComboEdit, CComboBoxEx>, public CItemEditor
{
public:
	DECLARE_WND_SUPERCLASS(NULL, CComboBoxEx::GetWndClassName())

	// bZ[W}bv
	BEGIN_MSG_MAP(CComboEdit)
		REFLECTED_COMMAND_CODE_HANDLER_EX(CBN_SELENDCANCEL, OnSelEndCancel)
	END_MSG_MAP()

	// EChEnhԂ܂
	operator HWND() const
	{
		return m_hWnd;
	}
	// I̎擾
	int GetItemText(WTL::CString& strText) const
	{
		int i = GetCurSel();

		if ( i==CB_ERR ) return 0;
		
		GetLBText(i, strText);
		return strText.GetLength();
	}
	// C[Wԍ擾܂
	int GetItemImage() const
	{
		int i = GetCurSel();

		// G[̏ꍇ-1
		if ( i==CB_ERR ) return -1;
			
		// C[Wԍ̎擾
		COMBOBOXEXITEM cb_item = { 0 };
		cb_item.iItem = i;
		cb_item.mask = CBEIF_IMAGE;
		GetItem(&cb_item);

		return cb_item.iImage;
	}
	// ҏWJn܂
	void BeginItemEdit(const CRect& rect, LPCTSTR strText, int image=-1)
	{
		//EChEʒuƃTCYݒ
		MoveWindow(&rect);
		ShowWindow(SW_SHOW);
		SetFocus();
		
		// āAI܂
		if ( GetCount()>0 ) {
			
			int i = FindStringExact(0, strText);
			
			if ( i!=CB_ERR )
				SetCurSel(i);
			else
				SetCurSel(0);
		}
	}
	// ҏWI܂
	void EndItemEdit()
	{
		ShowWindow(SW_HIDE);
	}
	// ILZCxg
	void OnSelEndCancel(UINT uNotifyCode, int nID, CWindow wndCtl)
	{
		if ( IsWindowVisible() ) {
			::SendMessage(GetParent(), WM_USER_ENDITEMEDIT, NULL, NULL);
		}
	}
	// EChE쐬܂
	HWND InitItemEdit(HWND hWndParent)
	{
		Create(hWndParent, CRect(0, 0, 100, 100), _T("ComboEdit"), CBS_DROPDOWNLIST|WS_CHILD|WS_BORDER|WS_VSCROLL);
		SetFont( GetParent().GetFont() );

		return GetParent();
	}
};

//---------------------------------------------------------------------------
//
//	TuACeҏWXgr[
//
//---------------------------------------------------------------------------
template<class TListItem>
class CListViewPlus : public CWindowImpl<CListViewPlus<TListItem>, CListViewCtrl>
{
public:
	typedef	typename std::vector<TListItem>::iterator iterator;
	typedef	typename std::vector<TListItem>::const_iterator const_iterator;

private:
	int		mItem;
	int		mSubItem;
	int		mEditType;
	int		mDragItem;
	bool	mIsDrag;
	bool	mIsDblClick;
	CPoint	mDragAdj;
	
	CItemEditor*	mCurrentEditor;

	//ҏWpGfBbg
	CImageList	mDragImage;
	std::vector<TListItem>		mItems;
	std::map<int,CItemEditor*>	mEditor;

	// GfB^̃nh擾
	HWND GetEditorHandle()
	{
		return (mCurrentEditor!=NULL) ?
			(HWND)(*mCurrentEditor) : reinterpret_cast<HWND>(INVALID_HANDLE_VALUE);
	}

public:
//	DECLARE_WND_SUPERCLASS(NULL, CListViewCtrl::GetWndClassName())

	//bZ[W}bv
	BEGIN_MSG_MAP(CListViewPlus)
		MSG_WM_MOUSEMOVE(OnMouseMove)
		MSG_WM_LBUTTONUP(OnLButtonUp)
		MSG_WM_LBUTTONDBLCLK(OnLButtonDblClick)
		MSG_WM_VSCROLL(OnScroll)
		MSG_WM_HSCROLL(OnScroll)
		MESSAGE_HANDLER_EX(WM_USER_ENDITEMEDIT, OnEndItemEdit)
		REFLECTED_NOTIFY_CODE_HANDLER_EX(LVN_GETDISPINFO, OnGetDispInfo)
		REFLECTED_NOTIFY_CODE_HANDLER_EX(LVN_BEGINDRAG, OnBeginDrag)
		REFLECT_NOTIFICATIONS_HWND_FILTERED( GetEditorHandle() )
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

	// XN[ɕҏWLZ
	void OnScroll(UINT nSBCode, UINT nPos, CScrollBar pScrollBar)
	{
		::SendMessage(m_hWnd, WM_USER_ENDITEMEDIT, 0, 0);
		SetMsgHandled(FALSE);
	}
	//}EXړCxg
	void OnMouseMove(UINT nFlags, CPoint point)
	{
		//hbO
		if ( mIsDrag ) {

			//J[\ʒuACeʒu擾܂
			LVHITTESTINFO lvhit;
			lvhit.pt = point;
			SubItemHitTest(&lvhit);

			//}ʒuɃ}[N\܂
			LVINSERTMARK lvmk = { 0 };
			lvmk.cbSize = sizeof(LVINSERTMARK);
			lvmk.iItem = lvhit.iItem;
			lvmk.dwFlags = (mDragItem>lvhit.iItem) ? LVIM_AFTER : 0;
			SetInsertMark(&lvmk);

			point.Offset(mDragAdj);
			mDragImage.DragMove(point);
		}
		SetMsgHandled(FALSE);
	}
	//hbOIo
	void OnLButtonUp(UINT nFlags, CPoint point)
	{
		LVHITTESTINFO lvhit;
		WTL::CString str;

		if ( mIsDrag ) {

			//hbOI
			ReleaseCapture();
			mDragImage.DragLeave(m_hWnd);
			mDragImage.EndDrag();
			mDragImage.Destroy();
			
			//}}[N폜܂
			LVINSERTMARK lvmk = { 0 };
			lvmk.cbSize = sizeof(LVINSERTMARK);
			lvmk.iItem = -1;
			SetInsertMark(&lvmk);

			//J[\ʒuACeʒu擾܂
			lvhit.pt = point;
			SubItemHitTest(&lvhit);

			// ACehbv̒ʒm
			EndItemDrop(lvhit.iItem, mDragItem);
			UpdateWindow();
			mIsDrag = false;
		}

		SetMsgHandled(FALSE);
	}
	//hbO̊Jn
	LRESULT OnBeginDrag(LPNMHDR pnmh)
	{
		NMLISTVIEW* pnmlv = reinterpret_cast<NMLISTVIEW*>(pnmh);
		CPoint pt;
		CRect rect;
		
		//LȃACeCfbNX̏ꍇ
		mIsDrag = false;
		if ( pnmlv->iItem>=0 ) {

			//CfbNX擾āAC[W쐬܂
			mDragItem = pnmlv->iItem;
			mDragImage = CreateDragImage(pnmlv->iItem, &pnmlv->ptAction);
			
			//C[W̃ItZbgvZ܂
			GetItemRect(mDragItem, &rect, LVIR_BOUNDS);
			GetCursorPos(&pt);
			ScreenToClient(&pt);
			mDragAdj.SetPoint(rect.left - pt.x, rect.top - pt.y);

			//hbOJn܂
			if ( !mDragImage.IsNull() ) {

				mDragImage.BeginDrag(0, 0, 0);
				mDragImage.DragEnter(m_hWnd, pnmlv->ptAction);
				SetCapture();
				
				mIsDrag = true;
			}
		}

		return FALSE;
	}
	//\ẽR[obN
	LRESULT OnGetDispInfo(LPNMHDR pnmh)
	{
		LV_DISPINFO* plvdi = reinterpret_cast<LV_DISPINFO*>(pnmh);
		LVITEM& item(plvdi->item);

		// CfbNX̃`FbN
		if ( (int)mItems.size()>item.iItem ) {

			if ( item.mask & LVIF_TEXT )
				lstrcpyn(item.pszText, mItems[ item.iItem ].GetText(item.iSubItem), item.cchTextMax);

			if ( item.mask & LVIF_IMAGE )
				item.iImage = mItems[ item.iItem ].GetImage();
		}

		return TRUE;
	}
	//_uNbNōڕҏWJn
	void OnLButtonDblClick(UINT nFlags, CPoint point)
	{
		WTL::CString str;
		LVHITTESTINFO lvhit;
		CRect rect, icon;

		//J[\ʒuACeʒu擾܂
		lvhit.pt = point;
		mItem = SubItemHitTest(&lvhit);
		mSubItem = lvhit.iSubItem;
		if ( mItem<0 || mSubItem<0 ) return;
		
		//GfBbg̕\ʒu擾܂
		if ( mSubItem==0 ) {
			GetItemRect(mItem, &rect, LVIR_LABEL);
			
			// ACRݒ肳Ăꍇ́A\̈g
			if ( GetImageList(LVSIL_SMALL)!=NULL ) {
				GetItemRect(mItem, &icon, LVIR_ICON);
				rect |= icon;
			}
		}
		else
			GetSubItemRect(mItem, mSubItem, LVIR_LABEL, &rect);
		
		rect.left += 1;
		rect.bottom -= 1;

		//ACẽeLXg擾܂
		str = mItems[ mItem ].GetText( mSubItem );
		mEditType = mItems[ mItem ].GetEditType( mSubItem );
		std::map<int,CItemEditor*>::iterator it = mEditor.find(mEditType);
		
		if ( it==mEditor.end() )
			mCurrentEditor = NULL;

		else {
			mCurrentEditor = it->second;
			mCurrentEditor->BeginItemEdit(&rect, str);
		}

		SetMsgHandled(FALSE);
	}
	//ACeҏW̊
	LRESULT OnEndItemEdit(UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		WTL::CString str;

		if ( mCurrentEditor!=NULL ) {
			
			// eXV܂
			mCurrentEditor->GetItemText(str);
			mCurrentEditor->EndItemEdit();
			mItems[ mItem ].SetText(mSubItem, str);
			if ( mSubItem==0 )
				mItems[ mItem ].SetImage( mCurrentEditor->GetItemImage() );

			// ҏWʒm܂
			EndItemEdit(mItem, mSubItem);
			mEditType = -1;
			mCurrentEditor = NULL;

			// ACeĕ`悵܂
			RedrawItems(mItem, mItem);
			UpdateWindow();
		}
		return FALSE;
	}
//
//
	//RXgN^
	CListViewPlus()
	{
		mIsDrag = false;
		mIsDblClick = false;
		mCurrentEditor = NULL;
		mEditType = -1;
	}
	//fXgN^
	virtual ~CListViewPlus()
	{
		for ( unsigned int i=0; i<mEditor.size(); i++ )
			delete mEditor[i];
	}
//
//
	//ACe̒ǉ
	TListItem* AddListItem(const TListItem& item)
	{
		mItems.push_back(item);
		CListViewCtrl::SetItemCountEx(mItems.size(), LVSICF_NOINVALIDATEALL);
		return &mItems.back();
	}
	//ACȇ}
	TListItem* InsertListItem(const TListItem& item, unsigned int index)
	{
		std::vector<TListItem>::iterator it;
		it = mItems.insert(mItems.begin()+index, item);
		CListViewCtrl::SetItemCountEx(mItems.size(), LVSICF_NOINVALIDATEALL);
		return *it;
	}
	//ACe폜܂
	void DeleteListItem(unsigned int index)
	{
		mItems.erase(mItems.begin() + index);
		CListViewCtrl::SetItemCountEx(mItems.size(), LVSICF_NOINVALIDATEALL);
	}
	//ACeS폜܂
	void ClearListItem()
	{
		mItems.clear();
		CListViewCtrl::SetItemCountEx(0, LVSICF_NOINVALIDATEALL);
	}
	//ACef[^ANZX
	template<class index_type>
	TListItem& operator[](index_type index)
	{
		return mItems[index];
	}
	template<class index_type>
	const TListItem& operator[](index_type index) const
	{
		return mItems[index];
	}
	//ACeCe[^
	iterator begin()
	{
		return mItems.begin();
	}
	iterator end()
	{
		return mItems.end();
	}
	const_iterator begin() const
	{
		return mItems.begin();
	}
	const_iterator end() const
	{
		return mItems.end();
	}
	//ACe̕ύX
	BOOL SetItemCount(int nItems)
	{
		mItems.resize(nItems);
		return CListViewCtrl::SetItemCountEx(mItems.size(), LVSICF_NOINVALIDATEALL);
	}
	BOOL SetItemCountEx(int nItems, DWORD dwFlags)
	{
		mItems.resize(nItems);
		return CListViewCtrl::SetItemCountEx(mItems.size(), dwFlags);
	}
//
//
	//GfB^̒ǉ
	template<class _Editor>
	_Editor* AddEditor(int edit_type)
	{
		_Editor* edit = new _Editor();
		edit->InitItemEdit(*this);
		DeleteEditor(edit_type);
		mEditor[ edit_type ] = edit;
		return edit;
	}
	//GfB^̎擾
	template<class _Editor>
	_Editor* GetEditor(int edit_type)
	{
		std::map<int,CItemEditor*> it = mEditor.find(edit_type);
		return ( it!=mEditor.end() ) ? dynamic_cast<_Editor*>(*it) : NULL;
	}
	//GfB^̍폜
	void DeleteEditor(int edit_type)
	{
		std::map<int,CItemEditor*>::iterator it = mEditor.find(edit_type);
		if ( it!=mEditor.end() ) {
			delete it->second;
			mEditor.erase(it);
		}
	}
	//GfB^S폜܂
	void ClearEditor()
	{
		std::map<int,CItemEditor*>::iterator it;
		for ( it=mEditor.begin(); it!=mEditor.end(); it++ )
			delete (*it);
		mEditor.clear();
	}
//
//
	// ACeҏW̊ʒm
	virtual void EndItemEdit(int item, int sub_item) {}
	// ACehbv̒ʒm
	virtual void EndItemDrop(int dest, int item) {}
};
