#pragma once
#include "ListItem.h"
#include "ParamInterface.h"
#include "MelonUnits.h"
#include "TouchStone.h"

#define COMP_SHUNTIND		0
#define COMP_CASCADEIND		1
#define COMP_SHUNTCAP		2
#define COMP_CASCADECAP		3
#define COMP_NPORTTERM		4
#define COMP_TRLINE			5
#define COMP_SHUNTSTUB		6
#define COMP_OPENSTUB		7
//---------------------------------------------------------------------------
//
//	}b`OGgNX
//
//---------------------------------------------------------------------------
class CMatchingElem : public IParamInterface
{
public:
	typedef complex_array::size_type size_type;
	typedef complex_array::iterator iterator;
	typedef complex_array::const_iterator const_iterator;

protected:
	SolutionPtr		mSolution;
	CMatchingElem*	mChain;

public:
	// RXgN^ƃfXgN^
	CMatchingElem(SolutionPtr sol, CMatchingElem* chain=NULL);
	virtual ~CMatchingElem(){}

	// f[^̒ڃANZX
	virtual std::complex<double>& operator[](size_type index) { return *(begin() + index); }
	virtual const std::complex<double>& operator[](size_type index) const  { return *(begin() + index); }
	SolutionPtr GetSolution() { return mSolution; }

	// Ce[^̎擾
	virtual iterator begin() = 0;
	virtual iterator end() = 0;
	virtual const_iterator begin() const = 0;
	virtual const_iterator end() const = 0;
	
	virtual const_iterator upper_bound(double freq);
	virtual const_iterator lower_bound(double freq);
	virtual const_iterator find(double freq);

	
	// `F[
	virtual void SetChain(CMatchingElem* chain);
	virtual CMatchingElem* GetChain() { return mChain; }

	// f[^̍XV
	virtual void Update()=0;
};


//---------------------------------------------------------------------------
//
//	NPortf[^I[
//
//---------------------------------------------------------------------------
class CNPortTerminal : public CMatchingElem
{
private:
	int		mPort;
	CNPortData		mNPort;
	complex_array	mReturn;

public:
	// RXgN^
	CNPortTerminal(const CNPortData& snp, int port);
	CNPortTerminal(const WTL::CString& nport_file, int port);

	// Ce[^̎擾
	iterator begin() { return mReturn.begin(); }
	iterator end() { return mReturn.end(); }
	const_iterator begin() const { return mReturn.begin(); }
	const_iterator end() const { return mReturn.end(); }

	// `F[
	void SetChain(CMatchingElem* chain) {}
	
	// p[^̓o
	void SetParam(IListItem& item);
	void GetParam(IListItem& item);
	int NumProperties() const { return 0; }
	void Update();
};

//---------------------------------------------------------------------------
//
//	ėpW萔fq
//
//---------------------------------------------------------------------------
class CLumpedElem : public CMatchingElem
{
protected:
	double			mValue;
	CMelonUnits		mUnit;
	complex_array	mReturn;

public:
	// RXgN^
	CLumpedElem(CMatchingElem* chain, double val=1e-9);

	// Ce[^̎擾
	iterator begin() { return mReturn.begin(); }
	iterator end() { return mReturn.end(); }
	const_iterator begin() const { return mReturn.begin(); }
	const_iterator end() const { return mReturn.end(); }

	// `F[
	void SetChain(CMatchingElem* chain);
	
	// p[^̓o
	void SetParam(IListItem& item);
	int NumProperties() const { return 0; }
};


//---------------------------------------------------------------------------
//
//	C_N^
//
//---------------------------------------------------------------------------
class CCascadedInd : public CLumpedElem
{
public:
	// RXgN^
	CCascadedInd(CMatchingElem* chain, double val=1e-9)
		: CLumpedElem(chain, val) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	LpV^
//
//---------------------------------------------------------------------------
class CCascadedCap : public CLumpedElem
{
public:
	// RXgN^
	CCascadedCap(CMatchingElem* chain, double val=1e-12)
		: CLumpedElem(chain, val) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	LpV^
//
//---------------------------------------------------------------------------
class CShuntedCap : public CLumpedElem
{
public:
	// RXgN^
	CShuntedCap(CMatchingElem* chain, double val=1e-12)
		: CLumpedElem(chain, val) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	C_N^
//
//---------------------------------------------------------------------------
class CShuntedInd : public CLumpedElem
{
public:
	// RXgN^
	CShuntedInd(CMatchingElem* chain, double val=1e-9)
		: CLumpedElem(chain, val) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	ėpz萔fq
//
//---------------------------------------------------------------------------
class CDistributedElem : public CMatchingElem
{
protected:
	double			mFreq;
	double			mImpedance;
	double			mELength;
	CMelonUnits		mFUnit;
	complex_array	mReturn;

public:
	// RXgN^
	CDistributedElem(CMatchingElem* chain);

	// Ce[^̎擾
	iterator begin() { return mReturn.begin(); }
	iterator end() { return mReturn.end(); }
	const_iterator begin() const { return mReturn.begin(); }
	const_iterator end() const { return mReturn.end(); }

	// `F[
	void SetChain(CMatchingElem* chain);
	
	// p[^̓o
	void SetParam(IListItem& item);
	
	// vpeB̓o
	void SetProperty(int index, IListItem& item);
	void GetProperty(int index, IListItem& item);

	// vpeB̐
	int NumProperties() const { return 3; }
};


//---------------------------------------------------------------------------
//
//	I[vX^u
//
//---------------------------------------------------------------------------
class COpenStub : public CDistributedElem
{
public:
	// RXgN^
	COpenStub(CMatchingElem* chain)
		: CDistributedElem(chain) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	ZX^u
//
//---------------------------------------------------------------------------
class CShuntStub : public CDistributedElem
{
public:
	// RXgN^
	CShuntStub(CMatchingElem* chain)
		: CDistributedElem(chain) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	`H
//
//---------------------------------------------------------------------------
class CTransmissionLine : public CDistributedElem
{
public:
	// RXgN^
	CTransmissionLine(CMatchingElem* chain)
		: CDistributedElem(chain) { Update(); }

	// p[^̏o
	void GetParam(IListItem& item);

	// S11XV܂
	void Update();
};


//---------------------------------------------------------------------------
//
//	t@Ng\bh
//
//---------------------------------------------------------------------------
CMatchingElem* CreateElement(LPCTSTR name, CMatchingElem* chain);
