#pragma once
#include "ParamInterface.h"

//---------------------------------------------------------------------------
//
//	`X^CC^[tFCX
//
//---------------------------------------------------------------------------
const DWORD STYLE_IMPGRID = 1<<0;
const DWORD STYLE_ADMGRID = 1<<1;
const DWORD STYLE_IMPTICK = 1<<2;
const DWORD STYLE_ADMTICK = 1<<3;

class CSmithStyle : public IParamInterface
{
public:
	typedef std::vector<double>::iterator grid_iterator;
	typedef std::vector<double>::const_iterator const_grid_iterator;

protected:
	double	mRadius;
	DWORD	mFormat;

	std::vector<double>		mRGrid;
	std::vector<double>		mXGrid;
	std::vector<double>		mGGrid;
	std::vector<double>		mBGrid;

public:
	// RXgN^ƃfXgN^
	CSmithStyle() :
		mRadius(100), mFormat(STYLE_IMPGRID |STYLE_ADMGRID |STYLE_IMPTICK) {}
	virtual ~CSmithStyle(){}
	
	// `X^C̃ANZX
	virtual Gdiplus::Pen* BoldGrid() = 0;
	virtual Gdiplus::Pen* AdmGrid() = 0;
	virtual Gdiplus::Pen* ImpGrid() = 0;
	virtual Gdiplus::SolidBrush* TextBrush() = 0;
	virtual Gdiplus::SolidBrush* BlackBrush() = 0;
	virtual Gdiplus::SolidBrush* WhiteBrush() = 0;
	virtual Gdiplus::Font* DefaultFont() = 0;

	// X~X`[g̔a
	void SetRadius(double r)	{ mRadius = r; }
	double GetRadius() const	{ return mRadius; }
	
	void SetFormat(DWORD fmt, bool sw);
	DWORD GetFormat() const		{ return mFormat; }
	bool IsFormat(DWORD fmt) const	{ return (bool)(mFormat & fmt); }

	// Cs[_XObh
	const_grid_iterator FirstRGrid() const	{ return mRGrid.begin(); }
	const_grid_iterator LastRGrid() const	{ return mRGrid.end(); }
	const_grid_iterator FirstXGrid() const	{ return mXGrid.begin(); }
	const_grid_iterator LastXGrid() const	{ return mXGrid.end(); }

	// Ah~^XObh
	const_grid_iterator FirstGGrid() const	{ return mGGrid.begin(); }
	const_grid_iterator LastGGrid() const	{ return mGGrid.end(); }
	const_grid_iterator FirstBGrid() const	{ return mBGrid.begin(); }
	const_grid_iterator LastBGrid() const	{ return mBGrid.end(); }
};


//---------------------------------------------------------------------------
//
//	W`X^CNX
//
//---------------------------------------------------------------------------
class CBasicSmithStyle : public CSmithStyle
{
private:
	Gdiplus::Pen*	mBoldGrid;
	Gdiplus::Pen*	mAdmGrid;
	Gdiplus::Pen*	mImpGrid;

	Gdiplus::SolidBrush*	mTextBrush;
	Gdiplus::SolidBrush*	mBlackBrush;
	Gdiplus::SolidBrush*	mWhiteBrush;
	Gdiplus::Font*      	mDefaultFont;

	void init_GridList();

	void get_pen_property(Gdiplus::Pen* pen, IListItem& item);

public:
	// RXgN^ƃfXgN^
	CBasicSmithStyle();
	~CBasicSmithStyle();
	
	// `X^CANZX
	Gdiplus::Pen* BoldGrid()	{ return mBoldGrid; }
	Gdiplus::Pen* AdmGrid()		{ return mAdmGrid; }
	Gdiplus::Pen* ImpGrid()		{ return mImpGrid; }
	Gdiplus::SolidBrush* TextBrush()	{ return mTextBrush; }
	Gdiplus::SolidBrush* BlackBrush()	{ return mBlackBrush; }
	Gdiplus::SolidBrush* WhiteBrush()	{ return mWhiteBrush; }
	Gdiplus::Font* DefaultFont()		{ return mDefaultFont; }
	
	// vpeB̓o
	void SetProperty(int index, IListItem& item);
	void GetProperty(int index, IListItem& item);

	// vpeB̐
	int NumProperties() const { return 8; }
};
