#pragma once
#include "stdafx.h"
#include "basic_matrixfunc.h"
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>

//---------------------------------------------------------------------------
//
//	\[Vf[^NX
//
//---------------------------------------------------------------------------
class CSolution
{
public:
	typedef std::vector<double>::iterator iterator;
	typedef std::vector<double>::const_iterator const_iterator;
	typedef std::vector<double>::size_type size_type;

private:
	std::vector<double>		mSweep;

public:
	// RXgN^ƃfXgN^
	CSolution(){}
	CSolution(double start, double stop, double step);
	virtual ~CSolution(){}

	// f[^ANZX
	double& operator[](size_type index) { return mSweep[index]; }
	const double& operator[](size_type index) const { return mSweep[index]; }
	size_type size() const		{ return mSweep.size(); }
	iterator begin()			{ return mSweep.begin(); }
	iterator end()				{ return mSweep.end(); }
	const_iterator begin() const{ return mSweep.begin(); }
	const_iterator end() const	{ return mSweep.end(); }
	iterator insert(iterator _where, double _val)	{ return mSweep.insert(_where, _val); }
	void clear()				{ mSweep.clear(); }
	void push_back(double _val)	{ mSweep.push_back(_val); }
	double back() const			{ return (mSweep.size()>0) ? mSweep.back()	: 0; }
	double front() const		{ return (mSweep.size()>0) ? mSweep.front()	: 0; }
};

typedef boost::shared_ptr<CSolution> SolutionPtr;
typedef boost::shared_ptr<const CSolution> const_SolutionPtr;

//---------------------------------------------------------------------------
//
//	HsNX
//
//---------------------------------------------------------------------------
class nport_fmt;
class CNPortData
{
public:
	typedef std::vector<complex_matrix>::iterator iterator;
	typedef std::vector<complex_matrix>::const_iterator const_iterator;
	typedef CSolution::size_type size_type;

private:

	int		mNumPorts;
	char	mParamType;
	SolutionPtr		pSolution;
	std::vector<complex_matrix>	mData;

	double get_frequency_unit( string fmt );
	void import_data( istream& ifs, double hz, nport_fmt& cvt );
	void diagonalStoZ0(complex_matrix& dest, const complex_matrix& s, double z0);

public:
	//RXgN^ƃfXgN^
	CNPortData(int num_ports=2);
	CNPortData(const WTL::CString& nport_file);
	CNPortData(const CNPortData& nport);
	CNPortData(const CNPortData& nport, const swap_matrix& adapter);
	virtual ~CNPortData(){}

	
	// Zq
	CNPortData& operator=(const CNPortData& nport);

	// f[^̍
	bool compose(int pos, const CNPortData& nport);
	bool compose(int pos, const CNPortData& nport, const swap_matrix& adapter);

	// f[^̍폜
	void clear();

	// Ce[^̎擾
	iterator begin()			{ return mData.begin(); }
	iterator end()				{ return mData.end(); }
	const_iterator begin() const{ return mData.begin(); }
	const_iterator end() const	{ return mData.end(); }

	// ef[^ANZX
	const complex_matrix& operator[](size_type index) const { return mData[index]; }
	complex_matrix& Interpolate(complex_matrix& mat, double freq) const;

	void NumPorts(int ports)	{ mNumPorts=ports; }
	int NumPorts() const		{ return mNumPorts; }
	char ParamType() const		{ return mParamType; }


	//Sp[^
	bool Z0transform(const CNPortData& z0src, const CNPortData& z0dest);
	bool Z0transform(const complex_matrix& z0src, const CNPortData& z0dest);
	bool Z0transform(const CNPortData& z0src, const complex_matrix& z0dest);
	bool Z0transform(const complex_matrix& z0src, const complex_matrix& z0dest);
	void ToSmatrix();

	//]u
	void conj_trans();

	// \[V̐ݒƎ擾
	SolutionPtr GetSolution() { return pSolution; }
	const_SolutionPtr GetSolution() const { return pSolution; }
	void SetSolution(SolutionPtr sol);

	//t@Co
	void import_data(const WTL::CString& path);
	void export_data(const WTL::CString& path);
};


// tH[}bgϊ֐IuWFNg
class nport_fmt
{
protected:
	double	Scale;

public:
	nport_fmt(double scale) : Scale(scale) {}
	virtual ~nport_fmt(){}
	virtual void operator()(double x, double y, std::complex<double>& out) {}
};

// real/imaginary `
class formatRI : public nport_fmt
{
public:
	formatRI(double scale) : nport_fmt(scale) {}
	void operator()(double x, double y, std::complex<double>& out)
	{
		out.real(x * Scale);
		out.imag(y * Scale);
	}
};

// magnitude/angle `
class formatMA : public nport_fmt
{
public:
	formatMA(double scale) : nport_fmt(scale) {}
	void operator()(double x, double y, std::complex<double>& out)
	{
		y = M_PI * y / 180.0;
		out.real(x * cos(y) * Scale);
		out.imag(x * sin(y) * Scale);
	}
};

// dB/angle `
class formatDB : public nport_fmt
{
public:
	formatDB(double scale) : nport_fmt(scale) {}
    void operator()(double x, double y, std::complex<double>& out)
	{
		y = M_PI * y / 180.0;
		x = pow(10.0, x/20.0);
		out.real(x * cos(y) * Scale);
		out.imag(x * sin(y) * Scale);
	}
};
